#include <Servo.h>
#include <SPI.h>
#include <SD.h>
#include <SoftwareSerial.h>

// Define the number of sensors and servos
const int numFlexSensors = 7;
const int numServos = 7;

// Define the flex sensor and servo pins
int masterFlexPins[numFlexSensors] = {A0, A1, A2, A3, A4, A5, A6};
int slaveFlexPins[numFlexSensors] = {A7, A8, A9, A10, A11, A12, A13};
int servoPins[numServos] = {2, 3, 4, 5, 6, 7, 8};

// Define the servo and flex sensor objects
Servo myservos[numServos];
int masterFlexValues[numFlexSensors];
int slaveFlexValues[numFlexSensors];
int servoAngles[numServos];

// Define the SD card and data file objects
File dataFile;
const int chipSelect = 10;

// Define the software serial object for Bluetooth communication
SoftwareSerial BTSerial(9, 11); // RX | TX

// Define the start and stop buttons
const int startButton = 12;
const int stopButton = 13;

// Define program variables
bool programRunning = false;
int numRepetitions = 0;
bool startButtonPressed = false;
bool stopButtonPressed = false;

void setup() {
  // Initialize the serial port for debugging
  Serial.begin(9600);

  // Initialize the servo objects
  for (int i = 0; i < numServos; i++) {
    myservos[i].attach(servoPins[i]);
  }

  // Initialize the SD card
  Serial.print("Initializing SD card...");
  if (!SD.begin(chipSelect)) {
    Serial.println("Card failed, or not present");
    return;
  }
  Serial.println("card initialized.");

  // Open the data file
  dataFile = SD.open("data.txt", FILE_WRITE);
  if (dataFile) {
    Serial.println("Data file opened.");
  } else {
    Serial.println("Error opening data file.");
  }

  // Initialize the start and stop buttons
  pinMode(startButton, INPUT_PULLUP);
  pinMode(stopButton, INPUT_PULLUP);

  // Initialize the Bluetooth serial communication
  BTSerial.begin(9600);
  Serial.println("Bluetooth serial communication initialized.");
}

void loop() {
  // read the start and stop buttons
  startButtonPressed = digitalRead(startButton) == LOW;
  stopButtonPressed = digitalRead(stopButton) == LOW;

  // if start button is pressed, start program
  if (startButtonPressed) {
    programRunning = true;
    Serial.println("Program started.");
  }

  // if stop button is pressed, stop program
  if (stopButtonPressed) {
    programRunning = false;
    Serial.println("Program stopped.");
  }

  // if program is running, move servo motors
if (programRunning) {
  for (int i = 0; i < numServos; i++) {
    servoAngles[i] = map(masterFlexValues[i], 0, 1023, 0, 180);
    servoAngles[i] = constrain(servoAngles[i], 0, 181);
    myservos[i].write(servoAngles[i]);
    slaveFlexValues[i] = analogRead(slaveFlexPins[i]);
  }
} else {
  for (int i = 0; i < numServos; i++) {
    myservos[i].write(0);
  }
}

// save data to SD card every 5 repetitions
if (numRepetitions % 5 == 0 && numRepetitions != 0) {
  dataFile.flush();
  Serial.println("Data saved to SD card.");
}

// delay before next iteration
delay(10);

// increment the number of repetitions
numRepetitions++;
}